unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  SysUtils, Types, Forms, Controls, Graphics, StdCtrls, ExtCtrls;

type
  TForm1 = class(TForm)
    AddPanelButton: TButton;
    ScrollBox1: TScrollBox;
    SequenceListBox: TListBox;
    procedure AddPanelButtonClick({%H-}Sender: TObject);
  private
    function GetColorForTag(aTag: PtrInt): TColor;
    procedure Exchange(panelSender, panelSource: TPanel);
    procedure PanelDragDrop(Sender, Source: TObject; {%H-}X, {%H-}Y: Integer);
    procedure PanelDragOver(Sender, Source: TObject; {%H-}X, {%H-}Y: Integer; {%H-}State: TDragState; var Accept: Boolean);
    procedure RefreshSequenceDisplay;
  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

procedure TForm1.AddPanelButtonClick(Sender: TObject);
const
  ID: PtrInt = 0;
begin
  with TPanel.Create(ScrollBox1) do
    begin                                  // panels added to array in creation order
      SetInitialBounds(0, ID*100, ScrollBox1.Width-2, 100);
      Tag := ID;
      Color := GetColorForTag(Tag);
      Font.Height := 24;
      Font.Style := [fsBold];
      Caption := Tag.ToString;
      BevelOuter := bvNone;
      DragMode := dmAutomatic;
      OnDragOver := @PanelDragOver;
      OnDragDrop  := @PanelDragDrop;
      Parent := ScrollBox1;
    end;
  RefreshSequenceDisplay;
  Inc(ID);
  if ID > 4 then
    AddPanelButton.Enabled := False;
end;

function TForm1.GetColorForTag(aTag: PtrInt): TColor;
begin
  case aTag of
    0: Exit(clBlue);
    1: Exit(clYellow);
    2: Exit(clMoneyGreen);
    3: Exit(clSilver);
    4: Exit(clRed);
  end;
end;

procedure TForm1.PanelDragOver(Sender, Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
begin
  Accept := (Sender is TPanel) and (Source is TPanel) and not Sender.Equals(Source);
end;

procedure TForm1.RefreshSequenceDisplay;
var
  i, idx: Integer;
  tops: TIntegerDynArray = Nil;

  function TopsHasValue(aTop: Integer; out Index: Integer): Boolean;
  var
    j: Integer;
  begin
    Result := False;
    Index := -1;
    for j := 0 to High(tops) do
      if tops[j] = aTop then
        begin
          Index := j;
          Exit(True);
        end;
  end;

begin
  for i := 0 to Pred(ScrollBox1.ControlCount) do
    begin
      if (ScrollBox1.Controls[i] is TPanel) then
        begin
          SetLength(tops, Succ(Length(tops)));
          tops[High(tops)] := TPanel(ScrollBox1.Controls[i]).Top;
        end;
    end;
  SequenceListBox.Clear;
  SequenceListBox.Items.Add('Panel IDs in order:');
  SequenceListBox.Items.Add('------------------');
  for i := 0 to ScrollBox1.Height do
    if TopsHasValue(i, idx) then
      SequenceListBox.Items.Add('Panel %d',[idx]);
end;

procedure TForm1.PanelDragDrop(Sender, Source: TObject; X, Y: Integer);
var
  senderPnl: TPanel absolute Sender;
  sourcePnl: TPanel absolute Source;
begin
  if (Sender is TPanel) and (Source is TPanel) then
    begin
      Exchange(senderPnl, sourcePnl);
      RefreshSequenceDisplay;
    end;
end;

procedure TForm1.Exchange(panelSender, panelSource: TPanel);
var
  tmp: Integer;
begin
  begin
    tmp := panelSource.Top;
    panelSource.Top := panelSender.Top;
    panelSender.Top := tmp;
  end;
end;

end.
